#!/usr/bin/perl
# Simple IRC bot to proxy message from this machine to server/channel(s) specified below.
use strict;
use warnings;

use POE;
use POE::Component::IRC::State;
use POE::Component::IRC::Plugin::AutoJoin;
use POE::Component::IRC::Plugin::FollowTail;
use POE::Component::IRC::Plugin::Connector;
use POSIX qw(mkfifo);

my $readpipe = '/tmp/check_mk_msgs.fifo';
my $nickname = 'CheckMK';
my $ircname  = 'CheckMK';
my $server = 'ix1.undernet.org';
my $ssl = 0;  # Set to 1 when connecting to an SSL-enabled server:port combo
my $port = 6667;
my $ipv6 = 1; # Set to 0 if your machine has only ancient internets ;)
my $debug = 0;
my $lag_o_meter = 1; # enable lag reporting every 60 seconds

my %channels = (
	# '#Secret' => 'secret_password',
	'#mychannel'   => '',
);

my %connect = (
	nick => $nickname,
	ircname => $ircname,
	server => $server,
	UseSSL => $ssl,
	port => $port,
	useipv6 => 1,
	debug => $debug,
);
my $irc = POE::Component::IRC::State->spawn( %connect) or die "Could not connect to $server! $!";
print "Created IRC object for server $server\n";

# Check if the pipe exists
unless (-e $readpipe)
{
	mkfifo($readpipe, 0770) or die "Could not create pipe for reading at $readpipe: $!";
}

POE::Session->create(
	package_states => [
		main => [ qw(_default _start irc_001 irc_join irc_tail_input irc_tail_error irc_tail_reset) ],
	],
	heap => { irc => $irc },
);

$poe_kernel->run();

sub _start
{
	my ($heap, $kernel) = @_[HEAP, KERNEL];
	my $irc = $heap->{irc};
	$irc->plugin_add('AutoJoin', POE::Component::IRC::Plugin::AutoJoin->new( Channels => \%channels, RejoinOnKick => 1, Rejoin_delay => 5, Retry_when_banned => 10 ));
	$heap->{connector} = POE::Component::IRC::Plugin::Connector->new(reconnect => 5, delay => 30); # wait 5 secs before reconnect, 30 secs between pings
	$irc->plugin_add( 'Connector' => $heap->{connector} );
	# $irc->yield(register => qw(join irc_tail_input));
	$irc->yield ( connect => \%connect );
	$kernel->delay( 'lag_o_meter' => 60 ) if ($lag_o_meter);
	$irc->yield(register => qw(all));
	$irc->yield(connect => { } );
	$irc->plugin_add( 'FollowTail' => POE::Component::IRC::Plugin::FollowTail->new(
			filename => $readpipe,
		));
	return;
}

sub irc_001 {
	my $sender = $_[SENDER];

	# Since this is an irc_* event, we can get the component's object by
	# accessing the heap of the sender. Then we register and connect to the
	# specified server.
	my $irc = $sender->get_heap();

	print "Connected to ", $irc->server_name(), "\n";

	# we join our channels
	# $irc->yield( join => $_ ) for keys %channels;
	return;
}

sub irc_join {
	my $nick = (split /!/, $_[ARG0])[0];
	my $channel = $_[ARG1];
	my $irc = $_[SENDER]->get_heap();

	# only send the message if we were the one joining
	if ($nick eq $irc->nick_name()) {
		$irc->yield(privmsg => $channel, $nickname.' reporting for duty!');
	}
}

sub irc_tail_input {
	my ($kernel, $sender, $filename, $input) = @_[KERNEL, SENDER, ARG0, ARG1];
	print "Input [$filename]: $input\n";
	$kernel->post( $sender, 'privmsg', $_, $input ) for keys %channels;
	return;
}

sub irc_tail_error {
	my ($kernel, $sender, $filename, $errnum, $errstring) = @_[KERNEL, SENDER, ARG0 .. ARG2];
	# print "$filename: ERROR: $errnum $errstring\n";
	#$kernel->post( $sender, 'privmsg', $_, "$filename: ERROR: $errnum $errstring" ) for @channels;
	# $irc->plugin_del( 'FollowTail' );
	return;
}

sub irc_tail_reset {
	my ($kernel, $sender, $filename) = @_[KERNEL, SENDER, ARG0];
	print "Reset [$filename]\n";
	# $kernel->post( $sender, 'privmsg', $_, "$filename: RESET EVENT" ) for keys %channels;
	return;
}

sub _default {
	my ($event, $args) = @_[ARG0 .. $#_];
	my @output = ( "$event: " );

	for my $arg (@$args) {
		if ( ref $arg eq 'ARRAY' ) {
			push( @output, '[' . join(', ', @$arg ) . ']' );
		}
		else {
			push ( @output, "'".($arg//'undef')."'" );
		}
	}
	# print join ' ', @output, "\n";
	return;
}

sub lag_o_meter {
	my ($kernel,$heap) = @_[KERNEL,HEAP];
	print 'Time: ' . time() . ' Lag: ' . $heap->{connector}->lag() . "\n";
	$kernel->delay( 'lag_o_meter' => 60 );
	return;
}
